part of stream_binder;

abstract class JsonType {
  String get type_name;
  String get encoder_name;
  String get decoder_name;
  String get encoder;
  String get decoder;

}

class GeneratedType implements JsonType {

  final Map<String, JsonType> params;
  final String type_name;
  GeneratedType(this.type_name, this.params);
  String get decoder_name => "_${type_name}Decoder";
  String get decoder => new _GeneratedTypeDecoderCreator(this).decoder;
  String get encoder_name => "_${type_name}Encoder";
  String get encoder => new _GeneratedTypeEncoderCreator(this).encoder;

}

abstract class PrimitiveType implements JsonType {

}

class BoolType implements PrimitiveType {

  BoolType._();
  String get type_name => "bool";
  String get encoder_name => "_BoolEncoder";
  String get decoder_name => "_BoolDecoder";
  String get encoder => "bool $encoder_name(bool data) => data;";
  String get decoder => "bool $decoder_name(bool data) => data;";

}

class IntType implements PrimitiveType {

  IntType._();
  String get type_name => "int";
  String get encoder_name => "_IntEncoder";
  String get decoder_name => "_IntDecoder";
  String get encoder => "int $encoder_name(int data)=> data;";
  String get decoder => "int $decoder_name(int data)=> data;";

}

class StringType implements PrimitiveType {

  StringType._();
  String get type_name => "String";
  String get encoder_name => "_StringEncoder";
  String get decoder_name => "_StringDecoder";
  String get encoder => "String $encoder_name(String data) => data;";
  String get decoder => "String $decoder_name(String data) => data;";

}

class DateTimeType implements PrimitiveType {

  DateTimeType._();
  String get type_name => "DateTime";
  String get encoder_name => "_DateTimeEncoder";
  String get decoder_name => "_DateTimeDecoder";
  String get decoder => "DateTime $decoder_name(int data) => new DateTime.fromMillisecondsSinceEpoch(data);";
  String get encoder => "int $encoder_name(DateTime data) => data.millisecondsSinceEpoch;";

}

class ListType implements PrimitiveType {

  final JsonType sub_type;
  ListType(this.sub_type);
  String get sub_type_encoder_name => sub_type.encoder_name;
  String get sub_type_decoder_name => sub_type.decoder_name;
  String get sub_type_name => sub_type.type_name;
  String get type_name => "List<$sub_type_name>";
  String get encoder_name => "_${sub_type_name}ListTypeEncoder";
  String get decoder_name => "_${sub_type_name}ListTypeDecoder";
  String get encoder => "List $encoder_name(List data) => " "new List.generate(data.length, (i) => $sub_type_encoder_name(data[i]));";

  String get decoder => "$type_name $decoder_name(List data) => " "new List.generate(data.length,(i) => $sub_type_decoder_name(data[i]));";

}

StringType string_type = new StringType._();
IntType int_type = new IntType._();
DateTimeType datetime_type = new DateTimeType._();
BoolType bool_type = new BoolType._();
ListType string_list_type = new ListType(string_type);
ListType int_list_type = new ListType(int_type);
ListType datetime_list_type = new ListType(datetime_type);
ListType bool_list_type = new ListType(bool_type);